@@ -8,16 +8,14 @@
 #include <numeric>
 #include <sstream>
 #include <unordered_map>
+#include <mutex>
 
-#include <tbb/tbb.h>
-#include <tbb/mutex.h>
-#include <tbb/compat/thread>
-#include <tbb/spin_rw_mutex.h>
-#include <tbb/blocked_range.h>
-#include <tbb/parallel_sort.h>
-#include <tbb/parallel_reduce.h>
-#include <tbb/task_scheduler_init.h>
-#include <tbb/concurrent_unordered_set.h>
+#include "tbb/tbb.h"
+#include "oneapi/tbb/spin_rw_mutex.h"
+#include "oneapi/tbb/blocked_range.h"
+#include "oneapi/tbb/parallel_sort.h"
+#include "oneapi/tbb/parallel_reduce.h"
+#include "oneapi/tbb/concurrent_unordered_set.h"
 
 #include "junctionapi/junctionapi.h"
 
@@ -80,6 +78,7 @@ namespace TwoPaCo
 		{
 		public:
 			uint64_t operator()(const DnaString & dnaString) const
+
 			{
 				return dnaString.Hash();
 			}
@@ -94,7 +93,7 @@ namespace TwoPaCo
 			}
 		};
 
-		typedef tbb::concurrent_unordered_set<Occurence, OccurenceHash, OccurenceEquality> OccurenceSet;
+		typedef oneapi::tbb::concurrent_unordered_set<Occurence, OccurenceHash, OccurenceEquality> OccurenceSet;
 
 	public:
 
@@ -151,6 +150,7 @@ namespace TwoPaCo
 			for (const std::string & fn : fileName)
 			{
 				logStream << fn << std::endl;
+
 			}
 #ifdef LOGGING
 			std::ofstream logFile((tmpDirName + "/log.txt").c_str());
@@ -162,7 +162,7 @@ namespace TwoPaCo
 			std::ostream & logFile = std::cerr;
 #endif
 
-			tbb::mutex errorMutex;
+			std::mutex errorMutex;
 			std::unique_ptr<std::runtime_error> error;
 			
 			size_t edgeLength = vertexLength + 1;
@@ -183,7 +183,7 @@ namespace TwoPaCo
 			if (rounds > 1)
 			{
 				logStream << "Splitting the input kmers set..." << std::endl;
-				std::vector<std::unique_ptr<tbb::tbb_thread> > workerThread(threads);
+				std::vector<std::unique_ptr<std::thread> > workerThread(threads);
 				binCounter = new std::atomic<uint32_t>[BINS_COUNT];
 				std::fill(binCounter, binCounter + BINS_COUNT, 0);
 				ConcurrentBitVector bitVector(realSize);
@@ -195,7 +195,7 @@ namespace TwoPaCo
 						vertexLength,
 						*taskQueue[i],
 						binCounter);
-					workerThread[i].reset(new tbb::tbb_thread(worker));
+					workerThread[i].reset(new std::thread(worker));
 				}
 
 				DistributeTasks(fileName, edgeLength, taskQueue, error, errorMutex, logFile, offset, offsetFill);
@@ -222,6 +222,7 @@ namespace TwoPaCo
 			uint64_t high = realSize;
 			uint64_t lowBoundary = 0;
 			uint64_t totalFpCount = 0;
+
 			uint64_t verticesCount = 0;
 			std::ofstream bifurcationTempWrite((tmpDirName + "/bifurcations.bin").c_str(), ios::binary);
 			if (!bifurcationTempWrite)
@@ -265,7 +266,7 @@ namespace TwoPaCo
 					logStream << "Round " << round << ", " << low << ":" << high << std::endl;
 					logStream << "Pass\tFilling\tFiltering" << std::endl << "1\t";
 					{
-						std::vector<std::unique_ptr<tbb::tbb_thread> > workerThread(threads);
+						std::vector<std::unique_ptr<std::thread> > workerThread(threads);
 						for (size_t i = 0; i < workerThread.size(); i++)
 						{
 							FilterFillerWorker worker(low,
@@ -274,7 +275,7 @@ namespace TwoPaCo
 								std::ref(bitVector),
 								edgeLength,
 								std::ref(*taskQueue[i]));
-							workerThread[i].reset(new tbb::tbb_thread(worker));
+							workerThread[i].reset(new std::thread(worker));
 						}
 
 						DistributeTasks(fileName, edgeLength, taskQueue, error, errorMutex, logFile, offset, offsetFill);
@@ -290,9 +291,10 @@ namespace TwoPaCo
 					}
 
 					//bitVector.WriteToFile(filterDumpFile_);
-					tbb::mutex maskStorageMutex;
+					std::mutex maskStorageMutex;
 					std::ofstream maskStorage(CandidateMaskFileName(tmpDirName, round).c_str(), ios::binary);
 					if (!maskStorage)
+
 					{
 						throw std::runtime_error("Can't open a temporary file");
 					}
@@ -300,7 +302,7 @@ namespace TwoPaCo
 					logStream << time(0) - mark << "\t";
 					mark = time(0);
 					{
-						std::vector<std::unique_ptr<tbb::tbb_thread> > workerThread(threads);
+						std::vector<std::unique_ptr<std::thread> > workerThread(threads);
 						for (size_t i = 0; i < workerThread.size(); i++)
 						{
 							CandidateCheckingWorker worker(std::make_pair(low, high),
@@ -316,7 +318,7 @@ namespace TwoPaCo
 								error,
 								errorMutex);
 
-							workerThread[i].reset(new tbb::tbb_thread(worker));
+							workerThread[i].reset(new std::thread(worker));
 						}
 
 						DistributeTasks(fileName, vertexLength + 1, taskQueue, error, errorMutex, logFile, offset, offsetFill);
@@ -335,18 +337,18 @@ namespace TwoPaCo
 				}
 
 				mark = time(0);
-				tbb::spin_rw_mutex mutex;
+				oneapi::tbb::spin_rw_mutex mutex;
 				logStream << "2\t";
 				OccurenceSet occurenceSet(1 << 20);
 				{
-					tbb::mutex maskStorageMutex;
+					std::mutex maskStorageMutex;
 					inMaskStorage[round] = new std::ifstream(CandidateMaskFileName(tmpDirName, round).c_str(), ios::binary);
 					if (!(*inMaskStorage[round]))
 					{
 						throw std::runtime_error("Can't open a temporary file");
 					}
 
-					std::vector<std::unique_ptr<tbb::tbb_thread> > workerThread(threads);
+					std::vector<std::unique_ptr<std::thread> > workerThread(threads);
 					for (size_t i = 0; i < workerThread.size(); i++)
 					{
 						CandidateFinalFilteringWorker worker(hashFunctionSeed_,
@@ -361,9 +363,10 @@ namespace TwoPaCo
 							error,
 							errorMutex);
 
-						workerThread[i].reset(new tbb::tbb_thread(worker));
+						workerThread[i].reset(new std::thread(worker));
 					}
 
+
 					DistributeTasks(fileName, vertexLength + 1, taskQueue, error, errorMutex, logFile, offset, offsetFill);
 					for (size_t i = 0; i < taskQueue.size(); i++)
 					{
@@ -415,14 +418,14 @@ namespace TwoPaCo
 
 			mark = time(0);			
 			std::atomic<uint64_t> occurence;
-			tbb::mutex currentStubVertexMutex;
+			std::mutex currentStubVertexMutex;
 			std::atomic<uint64_t> currentPiece;			
 			uint64_t currentStubVertexId = verticesCount + 42;
 			JunctionPositionWriter posWriter(outFileNamePrefix);
 			occurence = currentPiece = 0;
 			{
-				tbb::mutex inMaskStorageMutex;
-				std::vector<std::unique_ptr<tbb::tbb_thread> > workerThread(threads);
+				std::mutex inMaskStorageMutex;
+				std::vector<std::unique_ptr<std::thread> > workerThread(threads);
 				for (size_t i = 0; i < workerThread.size(); i++)
 				{
 					EdgeConstructionWorker worker(vertexLength,
@@ -435,11 +438,12 @@ namespace TwoPaCo
 						currentStubVertexMutex,
 						tmpDirName,
 						inMaskStorage,
+
 						inMaskStorageMutex,
 						error,
 						errorMutex);
 
-					workerThread[i].reset(new tbb::tbb_thread(worker));
+					workerThread[i].reset(new std::thread(worker));
 				}
 
 				DistributeTasks(fileName, vertexLength + 1, taskQueue, error, errorMutex, logFile, offset, offsetFill);
@@ -490,7 +494,7 @@ namespace TwoPaCo
 			return ss.str();
 		}
 
-		static void ReportError(tbb::mutex & errorMutex, std::unique_ptr<std::runtime_error> & error, const std::string & msg)
+		static void ReportError(std::mutex & errorMutex, std::unique_ptr<std::runtime_error> & error, const std::string & msg)
 		{
 			errorMutex.lock();
 			if (error == 0)
@@ -506,6 +510,7 @@ namespace TwoPaCo
 		public:
 			InitialFilterFillerWorker(uint64_t binSize,
 				const VertexRollingHashSeed & hashFunction,
+
 				ConcurrentBitVector & filter,
 				size_t vertexLength,
 				TaskQueue & taskQueue,
@@ -577,6 +582,7 @@ namespace TwoPaCo
 		private:
 			uint64_t binSize;
 			const VertexRollingHashSeed & hashFunction;
+
 			ConcurrentBitVector & filter;
 			size_t vertexLength;
 			TaskQueue & taskQueue;
@@ -596,9 +602,9 @@ namespace TwoPaCo
 				std::atomic<uint64_t> & marksCount,
 				size_t round,
 				std::ofstream & maskStorage,
-				tbb::mutex & maskStorageMutex,
+				std::mutex & maskStorageMutex,
 				std::unique_ptr<std::runtime_error> & error,				
-				tbb::mutex & errorMutex) : bound(bound), hashFunction(hashFunction), bitVector(bitVector), vertexLength(vertexLength), taskQueue(taskQueue),
+				std::mutex & errorMutex) : bound(bound), hashFunction(hashFunction), bitVector(bitVector), vertexLength(vertexLength), taskQueue(taskQueue),
 				tmpDirectory(tmpDirectory), marksCount(marksCount), maskStorage(maskStorage), maskStorageMutex(maskStorageMutex), error(error), errorMutex(errorMutex), round(round)
 			{
 
@@ -648,6 +654,7 @@ namespace TwoPaCo
 											++inCount;
 										}
 
+
 										if (nextCh == posExtend || IsOutgoingEdgeInBloomFilter(hash, bitVector, nextCh))
 										{
 											++outCount;
@@ -699,9 +706,9 @@ namespace TwoPaCo
 			std::atomic<uint64_t> & marksCount;
 			size_t round;
 			std::ofstream & maskStorage;
-			tbb::mutex & maskStorageMutex;
+			std::mutex & maskStorageMutex;
 			std::unique_ptr<std::runtime_error> & error;
-			tbb::mutex & errorMutex;
+			std::mutex & errorMutex;
 		};
 
 
@@ -713,13 +720,14 @@ namespace TwoPaCo
 				size_t vertexLength,
 				TaskQueue & taskQueue,
 				OccurenceSet & occurenceSet,
-				tbb::spin_rw_mutex & mutex,
+				oneapi::tbb::spin_rw_mutex & mutex,
 				const std::string & tmpDirectory,
 				size_t round,
 				std::ifstream & maskStorage,
-				tbb::mutex & maskStorageMutex,
+				std::mutex & maskStorageMutex,
 				std::unique_ptr<std::runtime_error> & error,
-				tbb::mutex & errorMutex) : hashFunction(hashFunction), vertexLength(vertexLength), taskQueue(taskQueue), occurenceSet(occurenceSet),
+
+				std::mutex & errorMutex) : hashFunction(hashFunction), vertexLength(vertexLength), taskQueue(taskQueue), occurenceSet(occurenceSet),
 				mutex(mutex), tmpDirectory(tmpDirectory), round(round), maskStorage(maskStorage), maskStorageMutex(maskStorageMutex), error(error), errorMutex(errorMutex)
 			{
 
@@ -791,6 +799,7 @@ namespace TwoPaCo
 									}
 								}
 
+
 								if (pos + edgeLength < task.str.size())
 								{
 									char posPrev = task.str[pos];
@@ -812,13 +821,13 @@ namespace TwoPaCo
 			size_t vertexLength;
 			TaskQueue & taskQueue;
 			OccurenceSet & occurenceSet;
-			tbb::spin_rw_mutex & mutex;
+			oneapi::tbb::spin_rw_mutex & mutex;
 			const std::string & tmpDirectory;
 			size_t round;
 			std::ifstream & maskStorage;
-			tbb::mutex & maskStorageMutex;
+			std::mutex & maskStorageMutex;
 			std::unique_ptr<std::runtime_error> & error;
-			tbb::mutex & errorMutex;
+			std::mutex & errorMutex;
 		};
 
 		struct EdgeResult
@@ -856,12 +865,13 @@ namespace TwoPaCo
 				std::atomic<uint64_t> & currentPiece,
 				std::atomic<uint64_t> & occurences,
 				uint64_t & currentStubVertexId,
-				tbb::mutex & currentStubVertexMutex,
+				std::mutex & currentStubVertexMutex,
 				const std::string & tmpDirectory,
 				std::vector<std::ifstream*> & inMaskStorage,
-				tbb::mutex & inMaskStorageMutex,
+				std::mutex & inMaskStorageMutex,
 				std::unique_ptr<std::runtime_error> & error,
-				tbb::mutex & errorMutex) : vertexLength(vertexLength), taskQueue(taskQueue), bifStorage(bifStorage),
+
+				std::mutex & errorMutex) : vertexLength(vertexLength), taskQueue(taskQueue), bifStorage(bifStorage),
 				writer(writer), currentPiece(currentPiece), occurences(occurences), tmpDirectory(tmpDirectory),
 				error(error), errorMutex(errorMutex), currentStubVertexId(currentStubVertexId), currentStubVertexMutex(currentStubVertexMutex), 
 				inMaskStorage(inMaskStorage), inMaskStorageMutex(inMaskStorageMutex)
@@ -979,10 +989,10 @@ namespace TwoPaCo
 			std::atomic<uint64_t> & occurences;
 			const std::string & tmpDirectory;
 			std::unique_ptr<std::runtime_error> & error;
-			tbb::mutex & inMaskStorageMutex;
+			std::mutex & inMaskStorageMutex;
 			std::vector<std::ifstream*> & inMaskStorage;
-			tbb::mutex & errorMutex;
-			tbb::mutex & currentStubVertexMutex;
+			std::mutex & errorMutex;
+			std::mutex & currentStubVertexMutex;
 		};		
 		
 		class FilterFillerWorker
@@ -1102,7 +1112,7 @@ namespace TwoPaCo
 			size_t overlapSize,
 			std::vector<TaskQueuePtr> & taskQueue,
 			std::unique_ptr<std::runtime_error> & error,
-			tbb::mutex & errorMutex,
+			std::mutex & errorMutex,
 			std::ostream & logFile,
 			std::vector<size_t> & offset,
 			bool offsetFill)
